@@ -1,4 +1,5 @@
 # pylint: disable=too-many-lines
+import asyncio
 import copy
 import logging
 import mimetypes
@@ -75,9 +76,42 @@ def _hour_to_millisecond(hour: Optional[float]) -> Optional[int]:
     return int(hour * 3600_000) if hour is not None else None
 
 
-_ORGNIZATION_ID_FOR_AVAILABILITY = "___plannedWorktime___"
+_ORGANIZATION_ID_FOR_AVAILABILITY = "___plannedWorktime___"
 """予定稼働時間用の組織ID"""
 
+_JOB_CONCURRENCY_LIMIT = {
+    JobType.COPY_PROJECT: {JobType.GEN_INPUTS, JobType.GEN_TASKS, JobType.DELETE_PROJECT, JobType.MOVE_PROJECT},
+    JobType.GEN_INPUTS: {
+        JobType.COPY_PROJECT,
+        JobType.GEN_INPUTS,
+        JobType.GEN_TASKS,
+        JobType.GEN_INPUTS_LIST,
+        JobType.DELETE_PROJECT,
+        JobType.MOVE_PROJECT,
+    },
+    JobType.GEN_TASKS: {
+        JobType.COPY_PROJECT,
+        JobType.GEN_INPUTS,
+        JobType.GEN_TASKS,
+        JobType.GEN_ANNOTATION,
+        JobType.GEN_TASKS_LIST,
+        JobType.DELETE_PROJECT,
+        JobType.MOVE_PROJECT,
+    },
+    JobType.GEN_ANNOTATION: {JobType.GEN_TASKS, JobType.GEN_ANNOTATION, JobType.DELETE_PROJECT, JobType.MOVE_PROJECT},
+    JobType.GEN_TASKS_LIST: {JobType.GEN_TASKS, JobType.GEN_TASKS_LIST, JobType.DELETE_PROJECT, JobType.MOVE_PROJECT},
+    JobType.GEN_INPUTS_LIST: {
+        JobType.GEN_INPUTS,
+        JobType.GEN_INPUTS_LIST,
+        JobType.DELETE_PROJECT,
+        JobType.MOVE_PROJECT,
+    },
+    JobType.INVOKE_HOOK: {JobType.DELETE_PROJECT, JobType.MOVE_PROJECT},
+    JobType.DELETE_PROJECT: set(JobType),
+    JobType.MOVE_PROJECT: set(JobType),
+}
+"""同時に実行できないジョブを表しています。valueに指定されたジョブが1つ以上実行されている場合、keyに指定されたジョブは実行できません。"""
+
 
 class Wrapper:
     """
@@ -508,13 +542,49 @@ class Wrapper:
 
         return dest_obj
 
+    def __convert_annotation_specs_labels_v2_to_v1(
+        self, labels_v2: List[Dict[str, Any]], additionals_v2: List[Dict[str, Any]]
+    ) -> List[LabelV1]:
+        """アノテーション仕様のV2版からV1版に変換する。V1版の方が扱いやすいので。
+
+        Args:
+            labels_v2 (List[Dict[str, Any]]): V2版のラベル情報
+            additionals_v2 (List[Dict[str, Any]]): V2版の属性情報
+
+        Returns:
+            List[LabelV1]: V1版のラベル情報
+        """
+
+        def get_additional(additional_data_definition_id: str) -> Optional[Dict[str, Any]]:
+            return _first_true(
+                additionals_v2, pred=lambda e: e["additional_data_definition_id"] == additional_data_definition_id
+            )
+
+        def to_label_v1(label_v2) -> LabelV1:
+            additional_data_definition_id_list = label_v2["additional_data_definitions"]
+            new_additional_data_definitions = []
+            for additional_data_definition_id in additional_data_definition_id_list:
+                additional = get_additional(additional_data_definition_id)
+                if additional is not None:
+                    new_additional_data_definitions.append(additional)
+                else:
+                    raise ValueError(
+                        f"additional_data_definition_id='{additional_data_definition_id}' に対応する属性情報が存在しません。"
+                        f"label_id='{label_v2['label_id']}', label_name_en='{self.__get_label_name_en(label_v2)}'"
+                    )
+            label_v2["additional_data_definitions"] = new_additional_data_definitions
+            return label_v2
+
+        return [to_label_v1(label_v2) for label_v2 in labels_v2]
+
     def put_annotation_for_simple_annotation_json(
         self,
         project_id: str,
         task_id: str,
         input_data_id: str,
         simple_annotation_json: str,
-        annotation_specs_labels: List[LabelV1],
+        annotation_specs_labels: List[Dict[str, Any]],
+        annotation_specs_additionals: Optional[List[Dict[str, Any]]] = None,
     ) -> bool:
         """
         AnnoFabからダウンロードしたアノテーションzip配下のJSONと同じフォーマット（Simple Annotation)の内容から、アノテーションを登録する。
@@ -523,10 +593,15 @@ class Wrapper:
             project_id:
             task_id:
             input_data_id:
-            simple_annotation_json:
+            simple_annotation_json: AnnoFabからダウンロードしたアノテーションzip配下のJSONのパス
+            annotation_specs_labels: アノテーション仕様のラベル情報。annotation_specs_additionalsが指定されている場合はV2版、指定されない場合はV1版。
+            annotation_specs_additionals: アノテーション仕様の属性情報（V2版）
 
         Returns:
             True:アノテーション情報をした。False: 登録するアノテーション情報がなかったため、登録しなかった。
+
+        Notes:
+            2021/07以降、引数annotation_specs_labelsはV1版をサポートしなくなる予定です。
         """
         parser = SimpleAnnotationDirParser(Path(simple_annotation_json))
         annotation = parser.load_json()
@@ -537,9 +612,14 @@ class Wrapper:
             return False
 
         request_details: List[Dict[str, Any]] = []
+        annotation_specs_labels_v1 = (
+            self.__convert_annotation_specs_labels_v2_to_v1(annotation_specs_labels, annotation_specs_additionals)
+            if annotation_specs_additionals is not None
+            else annotation_specs_labels
+        )
         for detail in details:
             request_detail = self.__to_annotation_detail_for_request(
-                project_id, parser, detail, annotation_specs_labels
+                project_id, parser, detail, annotation_specs_labels_v1
             )
             if request_detail is not None:
                 request_details.append(request_detail)
@@ -1845,32 +1925,106 @@ class Wrapper:
         while True:
             job = get_latest_job()
             if job is None:
-                logger.debug("ジョブは存在しませんでした。")
+                logger.debug("job_type = %s のジョブは存在しませんでした。", job_type.value)
                 return True
             else:
                 if job_access_count == 0 and job["job_status"] != "progress":
-                    logger.debug("進行中のジョブはありませんでした。")
+                    logger.debug("job_type = %s である進行中のジョブはありませんでした。", job_type.value)
                     return True
 
             job_access_count += 1
 
             if job["job_status"] == "succeeded":
-                logger.debug("job_id = %s のジョブが成功しました。", job["job_id"])
+                logger.debug("job_id = %s, job_type = %s のジョブが成功しました。", job["job_id"], job_type.value)
                 return True
 
             elif job["job_status"] == "failed":
-                logger.info("job_id = %s のジョブが失敗しました。", job["job_id"])
+                logger.info("job_id = %s, job_type = %s のジョブが失敗しました。", job["job_id"], job_type.value)
                 return False
 
             else:
                 # 進行中
                 if job_access_count < max_job_access:
-                    logger.debug("job_id = %s のジョブが進行中です。%d 秒間待ちます。", job["job_id"], job_access_interval)
+                    logger.debug(
+                        "job_id = %s, job_type = %s のジョブが進行中です。%d 秒間待ちます。",
+                        job["job_id"],
+                        job_type.value,
+                        job_access_interval,
+                    )
                     time.sleep(job_access_interval)
                 else:
-                    logger.debug("job_id = %s のジョブに %d 回アクセスしましたが、完了しませんでした。", job["job_id"], job_access_count)
+                    logger.debug(
+                        "job_id = %s, job_type = %s のジョブに %d 回アクセスしましたが、完了しませんでした。",
+                        job["job_id"],
+                        job_type.value,
+                        job_access_count,
+                    )
                     return False
 
+    async def _job_in_progress_async(self, project_id: str, job_type: JobType) -> bool:
+        loop = asyncio.get_event_loop()
+        return await loop.run_in_executor(None, self.job_in_progress, project_id, job_type)
+
+    async def _wait_for_completion(
+        self, project_id: str, job_type: JobType, job_access_interval: int, max_job_access: int
+    ) -> bool:
+        loop = asyncio.get_event_loop()
+        return await loop.run_in_executor(
+            None, self.wait_for_completion, project_id, job_type, job_access_interval, max_job_access
+        )
+
+    def can_execute_job(self, project_id: str, job_type: JobType) -> bool:
+        """
+        ジョブが実行できる状態か否か。他のジョブが実行中で同時に実行できない場合はFalseを返す。
+
+        Args:
+            project_id: プロジェクトID
+            job_type: ジョブ種別
+
+        Returns:
+            ジョブが実行できる状態か否か
+        """
+        job_type_list = _JOB_CONCURRENCY_LIMIT[job_type]
+
+        # tokenがない場合、ログインが複数回発生するので、事前にログインしておく
+        if self.api.token_dict is None:
+            self.api.login()
+
+        # 複数のジョブに対して進行中かどうかを確認する
+        gather = asyncio.gather(*[self._job_in_progress_async(project_id, job_type) for job_type in job_type_list])
+        loop = asyncio.get_event_loop()
+        result = loop.run_until_complete(gather)
+
+        return all(not e for e in result)
+
+    def wait_until_job_is_executable(
+        self, project_id: str, job_type: JobType, job_access_interval: int = 60, max_job_access: int = 360
+    ):
+        """
+        ジョブが実行可能な状態になるまで待ちます。他のジョブが実行されているときは、他のジョブが終了するまで待ちます。
+
+        Args:
+            project_id: プロジェクトID
+            job_type: ジョブ種別
+            job_access_interval: ジョブにアクセスする間隔[sec]
+            max_job_access: ジョブに最大何回アクセスするか
+
+        """
+        job_type_list = _JOB_CONCURRENCY_LIMIT[job_type]
+        # tokenがない場合、ログインが複数回発生するので、事前にログインしておく
+        if self.api.token_dict is None:
+            self.api.login()
+
+        # 複数のジョブに対して進行中かどうかを確認する
+        gather = asyncio.gather(
+            *[
+                self._wait_for_completion(project_id, job_type, job_access_interval, max_job_access)
+                for job_type in job_type_list
+            ]
+        )
+        loop = asyncio.get_event_loop()
+        loop.run_until_complete(gather)
+
     #########################################
     # Public Method : Labor Control
     #########################################
@@ -1979,7 +2133,7 @@ class Wrapper:
             return labor
 
         query_params = {
-            "organization_id": _ORGNIZATION_ID_FOR_AVAILABILITY,
+            "organization_id": _ORGANIZATION_ID_FOR_AVAILABILITY,
             "account_id": account_id,
             "from": from_date,
             "to": to_date,
@@ -2119,7 +2273,7 @@ class Wrapper:
 
         """
 
-        organization_id = _ORGNIZATION_ID_FOR_AVAILABILITY
+        organization_id = _ORGANIZATION_ID_FOR_AVAILABILITY
 
         labor_list, _ = self.api.get_labor_control(
             query_params={"organization_id": organization_id, "account_id": account_id, "from": date, "to": date}
